@@ -2,6 +2,9 @@
 import re
 import datetime as dt
 from typing import Any, Dict, List, Tuple
+from pathlib import Path
+import pickle
+import os
 
 # External libraries
 import pandas as pd
@@ -27,8 +30,9 @@ class PaperBroker(API):
         Interval.HR_1,
         Interval.DAY_1,
     ]
+    req_keys = []
 
-    def __init__(self, account_path: str = None, commission_fee=0, streamer=None):
+    def __init__(self, path: str = None, streamer=None, commission_fee=0, save=False):
         """
         :commission_fee: When this is a number it is assumed to be a flat price
             on all buys and sells of assets. When this is a string formatted as
@@ -39,35 +43,92 @@ class PaperBroker(API):
             strings formatted as 'XX%'.
         """
 
+        super().__init__(path)
+
         self.stocks = []
         self.options = []
         self.cryptos = []
         self.orders = []
-
-        self.equity = 1000000.0
-        self.cash = 1000000.0
-        self.buying_power = 1000000.0
-        self.multiplier = 1
+        self.order_id = 0
         self.commission_fee = commission_fee
-        self.id = 0
+        self.save = save
+
         self.streamer = DummyStreamer() if streamer is None else streamer
 
-        if account_path:
-            with open(account_path, "r") as f:
-                account = yaml.safe_load(f)
-                self.equity = account["equity"]
-                self.cash = account["cash"]
-                self.buying_power = account["buying_power"]
-                self.multiplier = account["multiplier"]
+        if path is None:
+            self.save_path = "./save"
+        else:
+            self.save_path = path.replace("secret.yaml", "save")
 
-                for stock in account["stocks"]:
-                    self.stocks.append(stock)
+        # If there is a previously saved broker status, load it
+        save_file = Path(self.save_path)
+        if save_file.is_file() and save:
+            self._load_account()
 
-                for crypto in account["cryptos"]:
-                    self.cryptos.append(crypto)
+        if path is None or self.config is None:
+            self.config = {
+                "paper_equity": 1000000.0,
+                "paper_cash": 1000000.0,
+                "paper_buying_power": 1000000.0,
+                "paper_multiplier": 1,
+            }
+        else:
+            self.commission_fee = self.config["commission_fee"]
 
-    def setup(self, interval, trader_main=None):
-        super().setup(interval, trader_main)
+        self.equity = self.config["paper_equity"]
+        self.cash = self.config["paper_cash"]
+        self.buying_power = self.config["paper_buying_power"]
+        self.multiplier = self.config["paper_multiplier"]
+
+    def _load_account(self):
+        with open(self.save_path, "rb") as stream:
+            save_data = pickle.load(stream)
+
+            account = save_data["account"]
+            self.equity = account["equity"]
+            self.cash = account["cash"]
+            self.buying_power = account["buying_power"]
+            self.multiplier = account["multiplier"]
+
+            positions = save_data["positions"]
+            self.stocks = positions["stocks"]
+            self.options = positions["options"]
+            self.cryptos = positions["cryptos"]
+
+            orders = save_data["orders"]
+            self.orders = orders["orders"]
+            self.order_id = orders["order_id"]
+
+    def _save_account(self):
+        with open(self.save_path, "wb") as stream:
+            save_data = {
+                "account": {
+                    "equity": self.equity,
+                    "cash": self.cash,
+                    "buying_power": self.buying_power,
+                    "multiplier": self.multiplier,
+                },
+                "positions": {
+                    "stocks": self.stocks,
+                    "options": self.options,
+                    "cryptos": self.cryptos,
+                },
+                "orders": {
+                    "orders": self.orders,
+                    "order_id": self.order_id,
+                },
+            }
+            pickle.dump(save_data, stream)
+
+    def _delete_account(self):
+        try:
+            os.remove(self.save_path)
+            debugger.debug("Removed saved account file.")
+        except:
+            debugger.warning("Saved account file does not exists.")
+
+    def setup(self, stats, account, trader_main=None):
+        super().setup(stats, account, trader_main)
 
     # -------------- Streamer methods -------------- #
 
@@ -88,17 +149,18 @@ class PaperBroker(API):
     def fetch_crypto_positions(self) -> List[Dict[str, Any]]:
         return self.cryptos
 
-    def update_option_positions(self, positions) -> List[Dict[str, Any]]:
-        for r in self.options:
-            occ_sym = r["symbol"]
-            price = self.streamer.fetch_option_market_data(occ_sym)["price"]
+    # def update_option_positions(self, positions) -> List[Dict[str, Any]]:
+    #     for r in self.options:
+    #         occ_sym = r["symbol"]
+    #         price = self.streamer.fetch_option_market_data(occ_sym)["price"]
 
-            r["current_price"] = price
-            r["market_value"] = price * r["quantity"] * 100
-            r["cost_basis"] = r["avg_price"] * r["quantity"] * 100
+    #         r["current_price"] = price
+    #         r["market_value"] = price * r["quantity"] * 100
+    #         r["cost_basis"] = r["avg_price"] * r["quantity"] * 100
 
     def fetch_account(self) -> Dict[str, Any]:
         self.equity = self._calc_equity()
+        self._save_account()
         return {
             "equity": self.equity,
             "cash": self.cash,
@@ -106,13 +168,13 @@ class PaperBroker(API):
             "multiplier": self.multiplier,
         }
 
-    def fetch_stock_order_status(self, id: int) -> Dict[str, Any]:
-        ret = next(r for r in self.orders if r["id"] == id)
+    def fetch_stock_order_status(self, order_id: int) -> Dict[str, Any]:
+        ret = next(r for r in self.orders if r["order_id"] == order_id)
         sym = ret["symbol"]
 
         price = self.streamer.fetch_price_history(
             sym,
-            self.interval[sym]["interval"],
+            self.stats.watchlist_cfg[sym]["interval"],
             self.streamer.get_current_time() - dt.timedelta(days=7),
             self.streamer.get_current_time(),
         )[sym]["close"][-1]
@@ -177,13 +239,14 @@ class PaperBroker(API):
             self.equity = self._calc_equity()
 
         debugger.debug(f"Returning status: {ret}")
-        debugger.debug(f"Positions:\n{self.stocks}\n=========\n{self.cryptos}")
+        stocks, cryptos = self.stocks, self.cryptos
+        debugger.debug(f"Positions:\n{stocks}\n=========\n{cryptos}")
         debugger.debug(f"Equity:{self._calc_equity()}")
-
+        self._save_account()
         return ret
 
-    def fetch_option_order_status(self, id: int) -> Dict[str, Any]:
-        ret = next(r for r in self.orders if r["id"] == id)
+    def fetch_option_order_status(self, order_id: int) -> Dict[str, Any]:
+        ret = next(r for r in self.orders if r["order_id"] == order_id)
         sym = ret["base_symbol"]
         occ_sym = ret["symbol"]
 
@@ -264,13 +327,14 @@ class PaperBroker(API):
             self.equity = self._calc_equity()
 
         debugger.debug(f"Returning status: {ret}")
-        debugger.debug(f"Positions:\n{self.stocks}\n{self.options}\n{self.cryptos}")
+        stocks, cryptos = self.stocks, self.cryptos
+        debugger.debug(f"Positions:\n{stocks}\n{self.options}\n{cryptos}")
         debugger.debug(f"Equity:{self._calc_equity()}")
-
+        self._save_account()
         return ret
 
-    def fetch_crypto_order_status(self, id: int) -> Dict[str, Any]:
-        return self.fetch_stock_order_status(id)
+    def fetch_crypto_order_status(self, order_id: int) -> Dict[str, Any]:
+        return self.fetch_stock_order_status(order_id)
 
     def fetch_order_queue(self) -> List[Dict[str, Any]]:
         return self.orders
@@ -292,15 +356,15 @@ class PaperBroker(API):
             "quantity": quantity,
             "filled_qty": quantity,
             "limit_price": limit_price,
-            "id": self.id,
+            "order_id": self.order_id,
             "time_in_force": in_force,
             "status": "open",
             "side": side,
         }
 
         self.orders.append(data)
-        self.id += 1
-        ret = {"type": "STOCK", "id": data["id"], "symbol": data["symbol"]}
+        self.order_id += 1
+        ret = {"order_id": data["order_id"], "symbol": data["symbol"]}
         return ret
 
     def order_crypto_limit(
@@ -318,15 +382,15 @@ class PaperBroker(API):
             "quantity": quantity,
             "filled_qty": quantity,
             "limit_price": limit_price,
-            "id": self.id,
+            "order_id": self.order_id,
             "time_in_force": in_force,
             "status": "open",
             "side": side,
         }
 
         self.orders.append(data)
-        self.id += 1
-        ret = {"type": "CRYPTO", "id": data["id"], "symbol": data["symbol"]}
+        self.order_id += 1
+        ret = {"order_id": data["order_id"], "symbol": data["symbol"]}
         return ret
 
     def order_option_limit(
@@ -335,7 +399,7 @@ class PaperBroker(API):
         symbol: str,
         quantity: float,
         limit_price: float,
-        type: str,
+        option_type: str,
         exp_date: dt.datetime,
         strike: float,
         in_force: str = "gtc",
@@ -343,10 +407,10 @@ class PaperBroker(API):
 
         data = {
             "type": "OPTION",
-            "symbol": self.data_to_occ(symbol, exp_date, type, strike),
+            "symbol": data_to_occ(symbol, exp_date, option_type, strike),
             "quantity": quantity,
             "filled_qty": 0,
-            "id": self.id,
+            "order_id": self.order_id,
             "time_in_force": in_force,
             "status": "open",
             "side": side,
@@ -355,8 +419,8 @@ class PaperBroker(API):
         }
 
         self.orders.append(data)
-        self.id += 1
-        return {"type": "OPTION", "id": data["id"], "symbol": data["symbol"]}
+        self.order_id += 1
+        return {"order_id": data["order_id"], "symbol": data["symbol"]}
 
     # ------------- Helper methods ------------- #
 
@@ -372,13 +436,13 @@ class PaperBroker(API):
                 add = add * asset["multiplier"]
             e += add
         e += self.cash
-        return e
+        return float(e)
 
     def apply_commission(self, inital_price: float, commission_fee, side: str) -> float:
         if side == "buy":
-            f = lambda a, b: a + b
+            f = lambda a, b: float(a + b)
         elif side == "sell":
-            f = lambda a, b: a - b
+            f = lambda a, b: float(a - b)
 
         if type(commission_fee) in (int, float):
             return f(inital_price, commission_fee)
@@ -389,7 +453,7 @@ class PaperBroker(API):
                 commission_fee = inital_price * 0.01 * float(match.group(1))
                 return f(inital_price, commission_fee)
             raise Exception(
-                f"`commission_fee` {commission_fee} not valid, must match this regex expression: {pattern}"
+                f"`commission_fee` {commission_fee} not valid must match this regex expression: {pattern}"
             )
         elif type(commission_fee) is dict:
             return self.apply_commission(inital_price, commission_fee[side], side)
