@@ -16,30 +16,28 @@ from harvest.utils import *
 class PolygonStreamer(API):
 
     interval_list = [Interval.MIN_1, Interval.MIN_5, Interval.HR_1, Interval.DAY_1]
+    req_keys = ["polygon_api_key"]
 
     def __init__(self, path: str = None, is_basic_account: bool = False):
         super().__init__(path)
+
+        if self.config is None:
+            raise Exception(
+                f"Account credentials not found! Expected file path: {path}"
+            )
+
         self.basic = is_basic_account
 
-    def setup(self, interval, trader_main=None):
-        self.watch_stock = []
-        self.watch_crypto = []
-
-        for sym in interval:
-            if is_crypto(sym):
-                self.watch_crypto.append(sym)
-            else:
-                self.watch_stock.append(sym)
-
+    def setup(self, stats, account, trader_main=None):
+        super().setup(stats, account, trader_main)
         self.option_cache = {}
-        super().setup(interval, trader_main)
 
     def exit(self):
         self.option_cache = {}
 
     def main(self):
         df_dict = {}
-        combo = self.watch_stock + self.watch_crypto
+        combo = self.stats.watchlist_cfg.keys()
         if self.basic and len(combo) > 5:
             debugger.error(
                 "Basic accounts only allow for 5 API calls per minute, trying to get data for more than 5 assets! Aborting."
@@ -189,7 +187,7 @@ class PolygonStreamer(API):
             timespan=timespan,
             start=start_str,
             end=end_str,
-            api_key=self.config["api_key"],
+            api_key=self.config["polygon_api_key"],
         )
 
         response = json.load(urllib.request.urlopen(request))
@@ -252,7 +250,7 @@ class PolygonStreamer(API):
         w.println(f"All steps are complete now 🎉. Generating {path}...")
 
         d = {
-            "api_key": f"{api_key}",
+            "polygon_api_key": f"{api_key}",
         }
 
         with open(path, "w") as file:
