@@ -1,20 +1,22 @@
 from typing import List
-from logging import error
+from harvest.utils import debugger
+
 
 class Plugin:
     """
     As the name implies, it implements funtionalities that the Trader class
-    alone cannot support, like retrieving earning report dates.  
+    alone cannot support, like retrieving earning report dates.
 
     The name of the plugin will follow the format:
         {Name of source}{Functionality added}Plugin
     """
-    def __init__(self, name: str, dependencies: List[str]=[]):
+
+    def __init__(self, name: str, dependencies: List[str] = []):
         """
         :name: a string that will serve as the name of the plugin. Must
         be a valid python variable because in the algo class, this plugin
         will be an attribute of the algo and accessable via algo.name.
-        :dependencies: a list of python packages as strings that will be 
+        :dependencies: a list of python packages as strings that will be
         checked to ensure they are installed.
         Put all imports here in order to test if we have them. If not then
         error and call the installation function.
@@ -28,12 +30,14 @@ class Plugin:
         """
         Returns how to install the necessary prerequsites for this plugin.
         """
-        raise Exception("Not implemented")
+        raise NotImplementedError(
+            f"No installation steps for plugin: {type(self).__name__}"
+        )
 
     def _check_dependency(self, dep: str) -> None:
         try:
             exec(f"import {dep}")
         except ModuleNotFoundError as e:
-            error(f"Error importing module!\n {e}")
-            error(self.installation())
-            raise e
\ No newline at end of file
+            debugger.error(f"Error importing module!\n {e}")
+            debugger.error(self.installation())
+            raise e
