@@ -72,12 +72,13 @@ class LiveTrader:
 
         self.watchlist = []  # List of securities specified in this class
         self.algo = []  # List of algorithms to run.
-        self.order_queue = []  # Queue of unfilled orders.
 
         self.server = Server(self)  # Initialize the web interface server
 
         self.stats = Stats(None, tzlocal.get_localzone(), None)
 
+        self.storage.setup(self.stats)
+
         self.func = Functions(
             self.buy,
             self.sell,
@@ -87,10 +88,12 @@ class LiveTrader:
             self.get_asset_quantity,
             self.storage.load,
             self.storage.store,
+            self.storage.load_daytrade,
         )
 
         self.account = Account()
         self.positions = self.account.positions
+        self.orders = self.account.orders
 
     def _setup_debugger(self, debug):
         # Set up logger
@@ -124,12 +127,10 @@ class LiveTrader:
         if sync:
             self._setup_stats()
             for s in self.positions.stock_crypto:
-                self.watchlist.append(s["symbol"])
+                self.watchlist.append(s.symbol)
             for s in self.positions.option:
-                self.watchlist.append(s["base_symbol"])
-            for s in self.order_queue:
-                sym = s["base_symbol"] if s["type"] == "OPTION" else s["symbol"]
-                self.watchlist.append(sym)
+                self.watchlist.append(s.base_symbol)
+            self.watchlist.extend(self.orders.stock_crypto_symbols)
 
         # Remove duplicates in watchlist
         self.watchlist = list(set(self.watchlist))
@@ -142,15 +143,15 @@ class LiveTrader:
         if not self.watchlist:
             raise Exception("No securities were added to watchlist")
 
-        # Initialize the account
-        self._setup_account()
-        self.storage.init_performace_data(self.account.equity, self.streamer.timestamp)
-
-        self.broker.setup(self.stats, self.main)
+        self.broker.setup(self.stats, self.account, self.main)
         if self.broker != self.streamer:
             # Only call the streamer setup if it is a different
             # instance than the broker otherwise some brokers can fail!
-            self.streamer.setup(self.stats, self.main)
+            self.streamer.setup(self.stats, self.account, self.main)
+
+        # Initialize the account
+        self._setup_account()
+        self.storage.init_performace_data(self.account.equity, self.stats.timestamp)
 
         # Initialize the storage
         self._storage_init(all_history)
@@ -172,20 +173,10 @@ class LiveTrader:
 
         # Get any pending orders
         ret = self.broker.fetch_order_queue()
-        self.order_queue = ret
-        debugger.debug(f"Fetched orders:\n{self.order_queue}")
+        self.orders.init(ret)
+        debugger.debug(f"Fetched orders:\n{self.orders}")
 
-        # Get positions
-        stock_positions = self.broker.fetch_stock_positions()
-        option_positions = self.broker.fetch_option_positions()
-        crypto_positions = self.broker.fetch_crypto_positions()
-        self.positions.update(stock_positions, option_positions, crypto_positions)
-
-        debugger.debug(f"Fetched positions:\n{self.positions}")
-
-        # Update option stats
-        self.broker.update_option_positions(self.positions.option)
-        debugger.debug(f"Updated option positions:\n{self.positions.option}")
+        self._fetch_account_data()
 
     def _setup_params(self, watchlist, interval, aggregations):
         """
@@ -272,11 +263,14 @@ class LiveTrader:
         """
         Main loop of the Trader.
         """
-        # Periodically refresh access tokens
-        if self.stats.timestamp.hour % 12 == 0 and self.stats.timestamp.minute == 0:
-            self.streamer.refresh_cred()
+        # # Periodically refresh access tokens
+        # if self.stats.timestamp.hour % 12 == 0 and self.stats.timestamp.minute == 0:
+        #     self.streamer.refresh_cred()
 
         self.storage.add_performance_data(self.account.equity, self.stats.timestamp)
+        self.storage.add_calendar_data(
+            self.streamer.fetch_market_hours(self.stats.timestamp.date())
+        )
 
         # Save the data locally
         for sym in df_dict:
@@ -328,40 +322,37 @@ class LiveTrader:
         """Check to see if outstanding orders have been accepted or rejected
         and update the order queue accordingly.
         """
-        debugger.debug(f"Updating order queue: {self.order_queue}")
-        for i, order in enumerate(self.order_queue):
-            if "type" not in order:
-                raise Exception(f"key error in {order}\nof {self.order_queue}")
-            if order["type"] == "STOCK":
-                stat = self.broker.fetch_stock_order_status(order["id"])
-            elif order["type"] == "OPTION":
-                stat = self.broker.fetch_option_order_status(order["id"])
-            elif order["type"] == "CRYPTO":
-                stat = self.broker.fetch_crypto_order_status(order["id"])
-            debugger.debug(f"Updating status of order {order['id']}")
-            self.order_queue[i].update(stat)
+        debugger.debug(f"Updating order queue: {self.orders}")
+        for order in self.orders.orders:
+            typ = order.type
+            if typ == "STOCK":
+                stat = self.broker.fetch_stock_order_status(order.order_id)
+            elif typ == "OPTION":
+                stat = self.broker.fetch_option_order_status(order.order_id)
+            elif typ == "CRYPTO":
+                stat = self.broker.fetch_crypto_order_status(order.order_id)
+            debugger.debug(f"Updating status of order {order.order_id}")
+            order.update(stat)
+
+        debugger.debug(f"Updated order queue: {self.orders}")
 
-        debugger.debug(f"Updated order queue: {self.order_queue}")
-        new_order = []
         order_filled = False
-        for order in self.order_queue:
+        for order in self.orders.orders:
             # TODO: handle cancelled orders
-            if order["status"] == "filled":
+            if order.status == "filled":
                 order_filled = True
                 debugger.debug(
-                    f"Order {order['id']} filled at {order['filled_time']} at {order['filled_price']}"
+                    f"Order {order.order_id} filled at {order.filled_time} at {order.filled_price}"
                 )
                 self.storage.store_transaction(
-                    order["filled_time"],
+                    order.filled_time,
                     "N/A",
-                    order["symbol"],
-                    order["side"],
-                    order["quantity"],
-                    order["filled_price"],
+                    order.symbol,
+                    order.side,
+                    order.quantity,
+                    order.filled_price,
                 )
-            else:
-                new_order.append(order)
-        self.order_queue = new_order
+        self.orders.remove_non_open()
 
         # if an order was processed, update the positions and account info
         return order_filled
@@ -372,12 +363,13 @@ class LiveTrader:
         # API should also be called if load_watch is false, as there is a high chance
         # that data in local cache are not representative of the entire portfolio,
         # meaning total equity cannot be calculated locally
-        debugger.debug(f"Updating positions: {self.positions}")
+
+        debugger.debug(f"Got data: {df_dict}")
 
         for p in self.positions.stock_crypto:
             symbol = p.symbol
             if symbol in df_dict:
-                price = df_dict[symbol][symbol]["close"][-1]
+                price = df_dict[symbol][symbol]["close"]
             elif (
                 symbol not in self.watchlist
             ):  # handle cases when user has an asset not in watchlist
@@ -392,6 +384,8 @@ class LiveTrader:
 
         self.account.update()
 
+        debugger.debug(f"Updated positions: {self.positions}")
+
     def _fetch_account_data(self):
         stock_pos = [
             Position(p["symbol"], p["quantity"], p["avg_price"])
@@ -458,14 +452,14 @@ class LiveTrader:
         if ret is None:
             debugger.debug("BUY failed")
             return None
-        self.order_queue.append(ret)
+        self.orders.add_new_order(symbol, ret["order_id"], "buy", quantity, in_force)
         debugger.debug(f"BUY: {self.stats.timestamp}, {symbol}, {quantity}")
 
         return ret
 
     def sell(self, symbol: str, quantity: int, in_force: str, extended: bool):
         # Check how many of the given asset we currently own
-        owned_qty = self.get_asset_quantity(symbol, exclude_pending_sell=True)
+        owned_qty = self.get_asset_quantity(symbol, True, False)
         if quantity > owned_qty:
             debugger.debug(
                 "SELL failed: More quantities are being sold than currently owned."
@@ -485,13 +479,13 @@ class LiveTrader:
         if ret is None:
             debugger.debug("SELL failed")
             return None
-        self.order_queue.append(ret)
+        self.orders.add_new_order(symbol, ret["order_id"], "sell", quantity, in_force)
         debugger.debug(f"SELL: {self.stats.timestamp}, {symbol}, {quantity}")
         return ret
 
     # ================ Helper Functions ======================
     def get_asset_quantity(
-        self, symbol: str = None, include_pending_buy=False, exclude_pending_sell=False
+        self, symbol, include_pending_buy, include_pending_sell
     ) -> float:
         """Returns the quantity owned of a specified asset.
 
@@ -501,8 +495,8 @@ class LiveTrader:
         """
         if symbol is None:
             symbol = self.watchlist[0]
-
-        if typ := symbol_type(symbol) == "OPTION":
+        typ = symbol_type(symbol)
+        if typ == "OPTION":
             owned_qty = sum(
                 p.quantity for p in self.positions.option if p.symbol == symbol
             )
@@ -518,14 +512,14 @@ class LiveTrader:
         if include_pending_buy:
             owned_qty += sum(
                 o["quantity"]
-                for o in self.order_queue
+                for o in self.orders.orders
                 if o["symbol"] == symbol and o["side"] == "buy"
             )
 
-        if exclude_pending_sell:
+        if not include_pending_sell:
             owned_qty -= sum(
                 o["quantity"]
-                for o in self.order_queue
+                for o in self.orders.orders
                 if o["symbol"] == symbol and o["side"] == "sell"
             )
 
@@ -555,6 +549,23 @@ class LiveTrader:
         """
         self.watchlist = symbol if isinstance(symbol, list) else [symbol]
 
+    def day_trade_count(self):
+        # Get the 5-day trading window
+        calendar = self.storage.load_calendar()
+        open_days = calendar.loc[self.calendar["is_open"] == True]
+        if len(open_days) == 0:
+            return 0
+        elif len(open_days) < 5:
+            window_start = open_days.index[0]
+        else:
+            window_start = open_days.index[-5]
+
+        # Check how many daytrades occurred in the last 5 trading days
+        day_trades = self.storage.load_daytrade()
+        day_trades = day_trades.loc[day_trades["timestamp"] >= window_start]
+
+        return len(day_trades)
+
     def exit(self, signum, frame):
         # TODO: Gracefully exit
         debugger.debug("\nStopping Harvest...")
