@@ -57,6 +57,9 @@ class Stats:
         self._timezone = timezone
         self._watchlist_cfg = watchlist_cfg
 
+    def __str__(self):
+        return f"{self.timestamp} {self.timezone} {self.watchlist_cfg}"
+
     @property
     def timestamp(self):
         return self._timestamp
@@ -93,6 +96,7 @@ class Functions:
         get_asset_quantity=None,
         load=None,
         save=None,
+        load_daytrade=None,
     ):
         self.buy = buy
         self.sell = sell
@@ -102,12 +106,14 @@ class Functions:
         self.get_asset_quantity = get_asset_quantity
         self.load = load
         self.save = save
+        self.load_daytrade = load_daytrade
 
 
 class Account:
     def __init__(self, account_name=None):
         self._account_name = account_name
         self._positions = Positions()
+        self._orders = Orders()
 
         self._asset_value = 0
         self._cash = 0
@@ -116,6 +122,17 @@ class Account:
         self._buying_power = 0
         self._multiplier = 1
 
+    def __str__(self):
+        return (
+            f"Account:\t{self.account_name}\n"
+            + f"Cash:\t{self.cash}\n"
+            + f"Equity:\t{self.equity}\n"
+            + f"Buying Power:\t{self.buying_power}\n"
+            + f"Multiplier:\t{self.multiplier}\n\n"
+            + f"Positions:\n{self.positions}\n\n"
+            + f"Orders:\n{self.orders}"
+        )
+
     def init(self, dict):
         self._equity = dict["equity"]
         self._cash = dict["cash"]
@@ -134,6 +151,10 @@ class Account:
     def positions(self):
         return self._positions
 
+    @property
+    def orders(self):
+        return self._orders
+
     @property
     def equity(self):
         return self._equity
@@ -151,19 +172,176 @@ class Account:
         return self._multiplier
 
 
+class Orders:
+    def __init__(self):
+        self._orders = []
+
+    def __str__(self):
+        return "\n".join(str(order) for order in self._orders)
+
+    def init(self, orders):
+        self._orders = [Order(**o) for o in orders]
+
+    @property
+    def orders(self):
+        return self._orders
+
+    def get_order(self, order_id):
+        for o in self._orders:
+            if o.order_id == order_id:
+                return o
+
+    def add_new_order(self, symbol, order_id, side, quantity, time_in_force):
+        self._orders.append(Order(symbol, order_id, side, quantity, time_in_force))
+
+    def remove_non_open(self):
+        self._orders = list(filter(lambda x: x.status == "open", self._orders))
+
+    @property
+    def symbols(self):
+        return [o.symbol for o in self._orders]
+
+    @property
+    def stock_crypto_symbols(self):
+        return [o.base_symbol if o.type == "OPTION" else o.symbol for o in self._orders]
+
+
+class Order:
+    def __init__(self, symbol, order_id, side, quantity, time_in_force):
+        self._symbol = symbol
+        self._order_id = order_id
+        self._type = symbol_type(symbol)
+        self._side = side
+        self._time_in_force = time_in_force
+        self._quantity = quantity
+
+        self._status = None
+        self._filled_time = None
+        self._filled_price = None
+        self._filled_quantity = None
+
+    def __str__(self):
+        return f"{self.order_id} {self.status} {self.symbol} {self.type} {self.filled_quantity}/{self.quantity} {self.time_in_force} {self.filled_time} {self.filled_price}"
+
+    @property
+    def symbol(self):
+        return self._symbol
+
+    @property
+    def type(self):
+        return self._type
+
+    @property
+    def quantity(self):
+        return self._quantity
+
+    @property
+    def filled_quantity(self):
+        return self._filled_quantity
+
+    @property
+    def order_id(self):
+        return self._order_id
+
+    @property
+    def time_in_force(self):
+        return self._time_in_force
+
+    @property
+    def status(self):
+        return self._status
+
+    @property
+    def filled_time(self):
+        return self._filled_time
+
+    @property
+    def filled_price(self):
+        return self._filled_price
+
+    @property
+    def side(self):
+        return self._side
+
+    def update(self, val):
+        self._filled_quantity = val["quantity"]
+        self._status = val["status"]
+        self._filled_price = val["filled_price"]
+        self._filled_time = val["filled_time"]
+
+
+class OptionOrder(Order):
+    def __init__(
+        self,
+        type,
+        symbol,
+        quantity,
+        filled_qty,
+        id,
+        time_in_force,
+        status,
+        side,
+        filled_time,
+        filled_price,
+        base_symbol,
+    ):
+        super().__init__(
+            type,
+            symbol,
+            quantity,
+            filled_qty,
+            id,
+            time_in_force,
+            status,
+            side,
+            filled_time,
+            filled_price,
+        )
+        self._base_symbol = base_symbol
+
+    def __str__(self):
+        s = super().__str__()
+        return f"{s} {self.base_symbol}"
+
+    @property
+    def base_symbol(self):
+        return self._base_symbol
+
+
 class Positions:
     def __init__(self, stock=[], option=[], crypto=[]):
         self._stock = stock
         self._option = option
         self._crypto = crypto
 
+        for p in self._stock + self._option:
+            setattr(self, p.symbol, p)
+        for p in self._crypto:
+            setattr(self, "c_" + p.symbol[1:], p)
+
     def update(self, stock=None, option=None, crypto=None):
-        if stock is not None:
-            self._stock = stock
-        if option is not None:
-            self._option = option
-        if crypto is not None:
-            self._crypto = crypto
+        current_symbols = [p.symbol for p in self.all]
+        self._stock = stock
+        self._option = option
+        self._crypto = crypto
+        new_symbols = [p.symbol for p in self.all]
+        for p in self._stock + self._option:
+            setattr(self, p.symbol, p)
+        for p in self._crypto:
+            setattr(self, "_" + p.symbol, p)
+
+        deleted_symbols = list(set(current_symbols) - set(new_symbols))
+        for s in deleted_symbols:
+            if symbol_type(s) == "CRYPTO":
+                delattr(self, "c_" + s[1:])
+            else:
+                delattr(self, s)
+
+    def get(self, symbol):
+        for p in self.all:
+            if p.symbol == symbol:
+                return p
+        return None
 
     @property
     def stock(self):
@@ -190,7 +368,12 @@ class Positions:
         return sum(p.value for p in self.all)
 
     def __str__(self):
-        return f"Positions: \n\tStocks: {self._stock}\n\tOptions: {self._option}\n\tCrypto: {self._crypto}"
+        return (
+            "Positions: \n"
+            + f"\tStocks : {'='.join(str(p) for p in self._stock)}\n"
+            + f"\tOptions: {'='.join(str(p) for p in self._option)}\n"
+            + f"\tCrypto : {'='.join(str(p) for p in self._crypto)}"
+        )
 
 
 class Position:
@@ -208,7 +391,7 @@ class Position:
         self._current_price = current_price
         self._value = self._current_price * self._quantity
         self._profit = self._value - self._avg_price * self._quantity
-        self._profit_percent = self._profit / self._avg_price
+        self._profit_percent = self._profit / (self._avg_price * self._quantity)
 
     def buy(self, quantity, price):
         self._avg_price = (self._avg_price * self._quantity + price * quantity) / (
@@ -235,6 +418,34 @@ class Position:
     def avg_price(self):
         return self._avg_price
 
+    @property
+    def asset_type(self):
+        return symbol_type(self._symbol)
+
+    @property
+    def current_price(self):
+        return self._current_price
+
+    @property
+    def profit(self):
+        return self._profit
+
+    @property
+    def profit_percent(self):
+        return self._profit_percent
+
+    def __str__(self):
+        return (
+            f"\n[{self._symbol}]\n"
+            + f" Quantity:\t{self._quantity}\n"
+            + f" Avg. Cost:\t${self._avg_price}\n"
+            + f" Price:  \t${self._current_price}\n"
+            + f" Value:  \t${self._value}\n"
+            + f" Profit:\t${self._profit}\n"
+            + f" Returns:\t{'▲' if self._profit_percent > 0 else '▼'}{self._profit_percent * 100}%\n"
+            + "─" * 50
+        )
+
 
 class OptionPosition(Position):
     def __init__(
@@ -247,10 +458,18 @@ class OptionPosition(Position):
         self._option_type = option_type
         self._multiplier = multiplier
 
+    @property
+    def symbol(self):
+        return self._symbol.replace(" ", "")
+
     @property
     def base_symbol(self):
         return self._base_symbol
 
+    @property
+    def value(self):
+        return self._value * self._multiplier
+
 
 def interval_enum_to_string(enum):
     try:
@@ -314,7 +533,7 @@ def symbol_type(symbol):
     """
     if len(symbol) > 6:
         return "OPTION"
-    elif symbol[0] == "@":
+    elif symbol[0] == "@" or symbol[:2] == "c_":
         return "CRYPTO"
     else:
         return "STOCK"
@@ -322,30 +541,31 @@ def symbol_type(symbol):
 
 def occ_to_data(symbol: str):
     original_symbol = symbol
-    debugger.debug(f"Converting {symbol} to data")
     try:
-        sym = ""
         symbol = symbol.replace(" ", "")
-        i = 0
-        while symbol[i].isalpha():
-            i += 1
+        i = re.search(r"[^A-Z ]", symbol).start()
         sym = symbol[:i]
         symbol = symbol[i:]
-        debugger.debug(f"{sym}, {symbol}")
-
         date = dt.datetime.strptime(symbol[:6], "%y%m%d")
-        debugger.debug(f"{date}, {symbol}")
         option_type = "call" if symbol[6] == "C" else "put"
-        debugger.debug(f"{option_type}, {symbol}")
         price = float(symbol[7:]) / 1000
-        debugger.debug(f"{price}, {symbol}")
+
         return sym, date, option_type, price
     except Exception as e:
-        debugger.error(f"Error parsing OCC symbol: {original_symbol}, {e}")
-        # return None, None, None, None
         raise Exception(f"Error parsing OCC symbol: {original_symbol}, {e}")
 
 
+def data_to_occ(symbol: str, date: dt.datetime, option_type: str, price: float):
+    """
+    Converts data into a OCC format string
+    """
+    occ = symbol  # + ((6 - len(symbol)) * " ")
+    occ += date.strftime("%y%m%d")
+    occ = occ + "C" if option_type == "call" else occ + "P"
+    occ += f"{int(price*1000):08}"
+    return occ
+
+
 # =========== DataFrame utils ===========
 
 
@@ -378,8 +598,6 @@ def aggregate_df(df, interval: Interval) -> pd.DataFrame:
 
 
 # ========== Date utils ==========
-
-
 def now() -> dt.datetime:
     """
     Returns the current time precise to the minute in the UTC timezone
@@ -412,20 +630,20 @@ def str_to_datetime(date: str) -> dt.datetime:
 
 
 def convert_input_to_datetime(datetime, timezone: ZoneInfo):
-
+    """
+    Converts input to a datetime object, with timezone set to the given timezone.
+    If timezone is None, the returned datetime object will be offset-naive.
+    """
     if datetime is None:
         return None
-    elif isinstance(datetime, Timestamp):
-        datetime = tz.localize(datetime.timestamp)
     elif isinstance(datetime, str):
         datetime = str_to_datetime(datetime)
-    elif isinstance(datetime, dt.datetime):
-        datetime = datetime.replace(tzinfo=timezone)
-    else:
+    elif not isinstance(datetime, dt.datetime):
         raise ValueError(f"Cannot convert {datetime} to datetime.")
 
-    datetime = datetime.replace(tzinfo=timezone)
-    datetime = datetime.astimezone(tz.utc)
+    if timezone is not None:
+        datetime = datetime.replace(tzinfo=timezone)
+        datetime = datetime.astimezone(tz.utc)
 
     return datetime
 
@@ -522,7 +740,7 @@ def mark_down(x):
 
 
 def is_crypto(symbol: str) -> bool:
-    return symbol[0] == "@"
+    return symbol_type(symbol) == "CRYPTO"
 
 
 ############ Functions used for testing #################
@@ -542,11 +760,3 @@ def gen_data(symbol: str, points: int = 50) -> pd.DataFrame:
     df.columns = pd.MultiIndex.from_product([[symbol], df.columns])
 
     return df
-
-
-def not_gh_action(func):
-    def wrapper(*args, **kwargs):
-        if "GITHUB_ACTIONS" in os.environ:
-            return
-        func(*args, **kwargs)
-        return wrapper
